import "package:kana_kit/kana_kit.dart";

const kanaKit = KanaKit();

/// 
/// A helper library for dealing with parsing Hiragana/Katakana, with the
/// aim of interacting with Sinsy's MusicXMLToLabel file
/// 
/// All of the glyphs in this file were derived from Sinsy's dictionary
/// 
class JapaneseUTF8 {
  static const BIG_CHARS = [
    "ク",
    "グ",
    "く",
    "ぐ",
    "ゔ",
    "ん",
    "を",
    "ゑ",
    "ゐ",
    "わ",
    "ゎ",
    "ろ",
    "れ",
    "る",
    "り",
    "ら",
    "よ",
    "ょ",
    "ゆ",
    "ゅ",
    "や",
    "ゃ",
    "も",
    "め",
    "む",
    "み",
    "ま",
    "ぽ",
    "ぼ",
    "ほ",
    "ぺ",
    "べ",
    "へ",
    "ぷ",
    "ぶ",
    "ふ",
    "ぴ",
    "び",
    "ひ",
    "ぱ",
    "ば",
    "は",
    "の",
    "ね",
    "ぬ",
    "に",
    "な",
    "ど",
    "と",
    "で",
    "て",
    "づ",
    "つ",
    "っ",
    "ぢ",
    "ち",
    "だ",
    "た",
    "ぞ",
    "そ",
    "ぜ",
    "せ",
    "ず",
    "す",
    "じ",
    "し",
    "ざ",
    "さ",
    "ご",
    "こ",
    "げ",
    "け",
    "ぎ",
    "き",
    "が",
    "か",
    "お",
    "ぉ",
    "え",
    "ぇ",
    "う",
    "ぅ",
    "い",
    "ぃ",
    "あ",
    "ぁ",
    "ヴ",
    "ン",
    "ヲ",
    "ヱ",
    "ヰ",
    "ワ",
    "ヮ",
    "ロ",
    "レ",
    "ル",
    "リ",
    "ラ",
    "ヨ",
    "ョ",
    "ユ",
    "ュ",
    "ヤ",
    "ャ",
    "モ",
    "メ",
    "ム",
    "ミ",
    "マ",
    "ポ",
    "ボ",
    "ホ",
    "ペ",
    "ベ",
    "ヘ",
    "プ",
    "ブ",
    "フ",
    "ピ",
    "ビ",
    "ヒ",
    "パ",
    "バ",
    "ハ",
    "ノ",
    "ネ",
    "ヌ",
    "ニ",
    "ナ",
    "ド",
    "ト",
    "デ",
    "テ",
    "ヅ",
    "ツ",
    "ッ",
    "ヂ",
    "チ",
    "ダ",
    "タ",
    "ゾ",
    "ソ",
    "ゼ",
    "セ",
    "ズ",
    "ス",
    "ジ",
    "シ",
    "ザ",
    "サ",
    "ゴ",
    "コ",
    "ゲ",
    "ケ",
    "ギ",
    "キ",
    "ガ",
    "カ",
    "オ",
    "ォ",
    "エ",
    "ェ",
    "ウ",
    "ゥ",
    "イ",
    "ィ",
    "ア",
    "ァ",
  ];

  static const SMALL_CHARS = [
    "ァ",
    "ィ",
    "ゥ",
    "ェ",
    "ォ",
    "ヮ",
    "ぁ",
    "ぃ",
    "ぅ",
    "ぇ",
    "ぉ",
    "ゎ",
    "ょ",
    "ゅ",
    "ゃ",
    "ョ",
    "ュ",
    "ャ"
  ];

  static final _smallBois = SMALL_CHARS.join("");
  static final _bigBois = BIG_CHARS.join("");
  static final _charRegex = RegExp("[" + _bigBois + "][" + _smallBois + "]?");

  /// Converts input in any form (romaji/hiragana/katakana) into an array of hiragana
  /// syllables (i.e. ra, ri, re, ryo, nye, sho, etc.)
  static List<String> alphabetToHiragana(String input) {
    String hiragana = kanaKit.toHiragana(input);

    return _charRegex.allMatches(hiragana).toList().map((RegExpMatch val) => val.group(0)).toList();
  }
}